@@ -51,9 +51,22 @@ enum {
   A_KCONFIGS,
   A_MENUSYMS,
   A_DEFAULTS,
+  A_SYMDEPS,
+  A_DEFDEPS,
+  A_EVERYYES,
   A_EVERYNO,
   A_EVERYDEF,
+  A_EVERYYESI,
+  A_AUTOCONF_FREE,
+  A_AUTOCONF_FEATURE_MODEL,
+  A_AUTOCONF_CNF,
   A_EXTRACT,
+  A_SEARCH,
+  A_DEPSYM,
+  A_IDEPSYM,
+  A_NONBOOLS,
+  A_BOOLS,
+  A_TRISTATE,
   A_DEPS,
   A_DUMP,
 };
@@ -75,47 +88,96 @@ static char *config_prefix = "CONFIG_";
 
 static bool enable_reverse_dependencies = true;
 
+bool check_symbol(struct symbol *, char *);
+bool find_dep(struct expr *, char *);
+void print_deps(struct symbol *);
+void print_deps_expr(struct expr *);
+bool defdeps(struct symbol *);
+bool defdeps_expr(struct expr *);
+bool depsym(struct symbol *);
+bool depsym_expr(struct expr *);
+bool idepsym(struct symbol *);
+bool idepsym_expr(struct expr *);
 bool is_symbol(struct symbol *);
 
 /* extern void bconf_parse(char *file); */
-static int expr_compare_type(enum expr_type t1, enum expr_type t2)
+extern int expr_compare_type(enum expr_type t1, enum expr_type t2);
+
+/*
+ * See whether the symbol's direct or reverse dependency expressions
+ * contain the configuration variable name.
+ */
+bool check_symbol(struct symbol *sym, char *name)
 {
-	if (t1 == t2)
-		return 0;
-	switch (t1) {
-	case E_LEQ:
-	case E_LTH:
-	case E_GEQ:
-	case E_GTH:
-		if (t2 == E_EQUAL || t2 == E_UNEQUAL)
-			return 1;
-	case E_EQUAL:
-	case E_UNEQUAL:
-		if (t2 == E_NOT)
-			return 1;
-	case E_NOT:
-		if (t2 == E_AND)
-			return 1;
-	case E_AND:
-		if (t2 == E_OR)
-			return 1;
-	case E_OR:
-		if (t2 == E_LIST)
-			return 1;
-	case E_LIST:
-		if (t2 == 0)
-			return 1;
-	default:
-		return -1;
-	}
-	printf("[%dgt%d?]", t1, t2);
-	return 0;
+  if (!sym->name || strlen(sym->name) == 0)
+    return false;
+  if (!strcmp(sym->name, name))
+    return true;
+  if (sym->searched)
+    return sym->depends;
+
+  // need to check for circular dependencies first.
+  sym->searched = true;
+  if (!sym->dir_dep.expr)
+    sym->depends = false;
+  else
+    sym->depends = find_dep(sym->dir_dep.expr, name);
+  if (sym->rev_dep.expr)
+    sym->depends = sym->depends || find_dep(sym->rev_dep.expr, name);
+
+  if (sym->depends)
+    printf("%s\n", sym->name);
+  return sym->depends;
 }
 
 /*
  * See whether an expression contains the configuration variable name.
  * Recursively search in symbols referenced in the expression.
  */
+bool find_dep(struct expr *e, char *name)
+{
+  struct expr *left, *right;
+  struct symbol *sym;
+
+  left = e->left.expr;
+  right = e->right.expr;
+  sym = e->left.sym;
+
+	switch (e->type) {
+	case E_SYMBOL:
+	case E_EQUAL:
+	case E_UNEQUAL:
+    return check_symbol(sym, name);
+
+	case E_NOT:
+    return find_dep(left, name);
+
+	case E_OR:
+    return find_dep(left, name) || find_dep(right, name);
+
+	case E_AND:
+    return find_dep(left, name) && find_dep(right, name);
+
+	/* case E_LIST: */
+  /*   //E_LIST is created in menu_finalize and is related to <choice> */
+  /*   printf("%s ", e->right.sym->name); */
+	/* 	if (e->left.expr) { */
+  /*     printf("^ "); */
+  /*     expr_to_bdd(e->left.expr); */
+	/* 	} */
+	/* 	break; */
+	/* case E_RANGE: */
+  /*   printf("["); */
+  /*   printf("%s ", e->left.sym->name); */
+  /*   printf("%s", e->right.sym->name); */
+  /*   printf("]"); */
+	/* 	break; */
+
+  default:
+    fprintf(stderr, "error: invalid expression type\n");
+    /* exit(1); */
+	}
+}
 
 void print_symbol_detail(FILE *out, struct symbol *sym, bool force_naked) {
   if (sym->name) {
@@ -264,7 +326,7 @@ void print_python_symbol_detail(FILE *out, struct symbol *sym, bool force_naked)
     } else if (strcmp(sym->name, "n") == 0) {
       fprintf(out, "0");
     } else if (S_UNKNOWN == sym->type) {
-      fprintf(out, "\"%s\"", sym->name);
+      fprintf(out, "%s", sym_escape_string_value(sym->name));
     } else {
       if (! force_naked) {
         fprintf(out, "%s%s", config_prefix, sym->name);
@@ -367,26 +429,6 @@ void print_python_expr(struct expr *e, FILE *out, enum expr_type prevtoken)
     fprintf(out, " and ");
     print_python_expr(e->right.expr, out, E_AND);
 		break;
-	case E_LTH:
-    print_python_symbol(out, e->left.sym);
-    fprintf(out, " < ");
-    print_python_symbol(out, e->right.sym);
-		break;
-	case E_LEQ:
-    print_python_symbol(out, e->left.sym);
-    fprintf(out, " <= ");
-    print_python_symbol(out, e->right.sym);
-		break;
-	case E_GTH:
-    print_python_symbol(out, e->left.sym);
-    fprintf(out, " > ");
-    print_python_symbol(out, e->right.sym);
-		break;
-	case E_GEQ:
-    print_python_symbol(out, e->left.sym);
-    fprintf(out, " >= ");
-    print_python_symbol(out, e->right.sym);
-		break;
 	case E_LIST:
     // TODO: this will break python parser
     //E_LIST is created in menu_finalize and is related to <choice>
@@ -415,6 +457,107 @@ void print_python_expr(struct expr *e, FILE *out, enum expr_type prevtoken)
 		fprintf(out, ")");
 }
 
+void print_clause_symbol(FILE *out, struct symbol *sym) {
+  if (sym->name) {
+    if (strcmp(sym->name, "y") == 0 ||
+        strcmp(sym->name, "m") == 0) {
+      fprintf(out, "1");
+    } else if (strcmp(sym->name, "n") == 0) {
+      fprintf(out, "0");
+    } else if (S_UNKNOWN == sym->type) {
+      fprintf(out, "0");
+    } else if (! (sym->depends)) {
+      fprintf(out, "0");
+    } else {
+      fprintf(out, "CONFIG_%s", sym->name);
+    }
+  } else {
+    // check for choice node
+    /* fprintf(stderr, "%x\n", sym); */
+    /* fprintf(stderr, "%d\n", sym->type); */
+    /* fprintf(stderr, "%d\n", (sym->type == S_BOOLEAN)); */
+    /* fprintf(stderr, "%d\n", (sym->prop->type)); */
+    fprintf(out, "1");
+  }
+}
+
+void print_clause(struct expr *e, FILE *out, enum expr_type prevtoken)
+{
+	if (expr_compare_type(prevtoken, e->type) > 0)
+		fprintf(out, "(");
+	switch (e->type) {
+	case E_SYMBOL:
+    print_clause_symbol(out, e->left.sym);
+		break;
+	case E_NOT:
+    fprintf(out, "!");
+    print_clause(e->left.expr, out, E_NOT);
+		break;
+	case E_EQUAL:
+    if (strcmp(e->right.sym->name, "y") == 0 ||
+        strcmp(e->right.sym->name, "m") == 0) {
+      print_clause_symbol(out, e->left.sym);
+    } else if (strcmp(e->right.sym->name, "n") == 0) {
+      fprintf(out, "!");
+      print_clause_symbol(out, e->left.sym);
+    } else {
+      // don't print (defined ... ) around config
+      print_clause_symbol(out, e->left.sym);
+      fprintf(out, "==");
+      print_clause_symbol(out, e->right.sym);
+    }
+		break;
+	case E_UNEQUAL:
+    if (strcmp(e->right.sym->name, "y") == 0 ||
+        strcmp(e->right.sym->name, "m") == 0) {
+      fprintf(out, "!");
+      print_clause_symbol(out, e->left.sym);
+    } else if (strcmp(e->right.sym->name, "n") == 0) {
+      print_clause_symbol(out, e->left.sym);
+    } else {
+      // don't print (defined ... ) around config
+      print_clause_symbol(out, e->left.sym);
+      fprintf(out, "!=");
+      print_clause_symbol(out, e->right.sym);
+    }
+		break;
+	case E_OR:
+    print_clause(e->left.expr, out, E_OR);
+    fprintf(out, " || ");
+    print_clause(e->right.expr, out, E_OR);
+		break;
+	case E_AND:
+    print_clause(e->left.expr, out, E_AND);
+    fprintf(out, " && ");
+    print_clause(e->right.expr, out, E_AND);
+		break;
+	case E_LIST:
+    //E_LIST is created in menu_finalize and is related to <choice>
+    print_clause_symbol(out, e->right.sym);
+    fprintf(out, " ");
+		if (e->left.expr) {
+      fprintf(out, "^ ");
+      print_clause(e->left.expr, out, E_LIST);
+		}
+		break;
+	case E_RANGE:
+    fprintf(out, "[");
+    print_clause_symbol(out, e->left.sym);
+    print_clause_symbol(out, e->right.sym);
+    fprintf(out, "]");
+		break;
+	/* default: */
+	/*   { */
+	/* 	char buf[32]; */
+	/* 	sprintf(buf, "<unknown type %d>", e->type); */
+	/* 	fn(data, NULL, buf); */
+	/* 	break; */
+	/*   } */
+	}
+	if (expr_compare_type(prevtoken, e->type) > 0)
+		fprintf(out, ")");
+}
+
 void print_choice_clauses(struct expr *e, struct symbol *sym, FILE *out)
 {
 	switch (e->type) {
@@ -497,6 +640,12 @@ bool never(struct symbol *sym)
   return false;
 }
 
+/* Negate idepsym.  Used for the --everyyesi action. */
+bool inverse(struct symbol *sym)
+{
+  return !idepsym(sym);
+}
+
 /* Check whether a configuration variable should be forced to off */
 bool check_forceoff(struct symbol *sym)
 {
@@ -720,6 +869,247 @@ void everyno(void)
 
 }
 
+/* Print the list of symbols that the given symbol depends on. */
+void print_deps(struct symbol *sym)
+{
+  if (!sym->name || strlen(sym->name) == 0)
+    return;
+  if (sym->searched)
+    return;
+
+  printf("%s,", sym->name);
+
+  sym->searched = true;
+  if (sym->dir_dep.expr)
+    print_deps_expr(sym->dir_dep.expr);
+  if (sym->rev_dep.expr)
+    print_deps_expr(sym->rev_dep.expr);
+}
+
+void print_deps_expr(struct expr *e)
+{
+  struct expr *left, *right;
+  struct symbol *sym;
+
+  left = e->left.expr;
+  right = e->right.expr;
+  sym = e->left.sym;
+
+	switch (e->type) {
+	case E_SYMBOL:
+	case E_EQUAL:
+	case E_UNEQUAL:
+    print_deps(sym);
+    return;
+
+	case E_NOT:
+    print_deps_expr(left);
+    return;
+
+	case E_OR:
+	case E_AND:
+    print_deps_expr(left);
+    print_deps_expr(right);
+    return;
+
+  default:
+    fprintf(stderr, "error: invalid expression type\n");
+    /* exit(1); */
+	}
+}
+
+bool defdeps(struct symbol *sym)
+{
+  if (!sym->name || strlen(sym->name) == 0)
+    return true;
+  if (sym->searched)
+    return sym->depends;
+
+  sym->searched = true;
+  if (is_default(sym)) {
+    sym->depends = true;
+    return true;
+  }
+
+  if (!sym->dir_dep.expr && !sym->rev_dep.expr) {
+    sym->depends = false;
+    return false;
+  }
+
+  sym->depends = true;
+  if (sym->dir_dep.expr)
+    sym->depends = sym->depends && defdeps_expr(sym->dir_dep.expr);
+  if (sym->rev_dep.expr)
+    sym->depends = sym->depends && defdeps_expr(sym->rev_dep.expr);
+
+  return sym->depends;
+}
+
+/*
+ * See whether an expression contains the configuration variable name.
+ * Recursively search in symbols referenced in the expression.
+ */
+bool defdeps_expr(struct expr *e)
+{
+  struct expr *left, *right;
+  struct symbol *sym;
+
+  left = e->left.expr;
+  right = e->right.expr;
+  sym = e->left.sym;
+
+	switch (e->type) {
+	case E_SYMBOL:
+	case E_EQUAL:
+	case E_UNEQUAL:
+    return false;
+
+	case E_NOT:
+    return defdeps_expr(left);
+
+	case E_OR:
+    return defdeps_expr(left) && defdeps_expr(right);
+
+	case E_AND:
+    return defdeps_expr(left) && defdeps_expr(right);
+
+  default:
+    fprintf(stderr, "error: invalid expression type\n");
+    /* exit(1); */
+	}
+}
+
+/* Returns true when the sym's dependencies only depend on
+ * non-architectural configuration variables.
+ */
+bool depsym(struct symbol *sym)
+{
+  if (!sym->name || strlen(sym->name) == 0)
+    return false;
+  if (sym->searched)
+    return sym->depends;
+
+  if (strcmp(sym->name, "y") == 0)
+    return false;
+  else if (strcmp(sym->name, "m") == 0)
+    return false;
+  else if (strcmp(sym->name, "n") == 0)
+    return false;
+
+  if (sym->dir_dep.expr == NULL && sym->rev_dep.expr == NULL
+      && is_symbol(sym)) {
+    sym->searched = true;
+    sym->depends = false;
+    return sym->depends;
+  }
+
+  sym->searched = true;
+  sym->depends = true;
+  if (sym->dir_dep.expr) {
+    sym->depends = sym->depends && depsym_expr(sym->dir_dep.expr);
+  }
+  if (sym->rev_dep.expr) {
+    sym->depends = sym->depends && depsym_expr(sym->rev_dep.expr);
+  }
+
+  return sym->depends;
+}
+
+bool depsym_expr(struct expr *e)
+{
+  struct expr *left, *right;
+  struct symbol *sym;
+
+  left = e->left.expr;
+  right = e->right.expr;
+  sym = e->left.sym;
+
+	switch (e->type) {
+	case E_SYMBOL:
+	case E_EQUAL:
+	case E_UNEQUAL:
+    return depsym(sym);
+
+	case E_NOT:
+    return false;
+
+	case E_OR:
+    return depsym_expr(left) && depsym_expr(right);
+
+	case E_AND:
+    return depsym_expr(left) || depsym_expr(right);
+
+  default:
+    fprintf(stderr, "error: invalid expression type\n");
+    /* exit(1); */
+	}
+}
+
+/* Returns true when the sym is selectable
+ */
+bool idepsym(struct symbol *sym)
+{
+  if (!sym->name || strlen(sym->name) == 0)
+    return true;
+  if (sym->searched)
+    return sym->depends;
+
+  if (strcmp(sym->name, "y") == 0)
+    return true;
+  else if (strcmp(sym->name, "m") == 0)
+    return true;
+  else if (strcmp(sym->name, "n") == 0)
+    return true;
+
+  if (sym->dir_dep.expr == NULL && sym->rev_dep.expr == NULL
+      && is_symbol(sym)) {
+    sym->searched = true;
+    sym->depends = true;
+    return sym->depends;
+  }
+
+  sym->searched = true;
+  sym->depends = false;
+  if (sym->dir_dep.expr) {
+    sym->depends = sym->depends || idepsym_expr(sym->dir_dep.expr);
+  }
+  if (sym->rev_dep.expr) {
+    sym->depends = sym->depends || idepsym_expr(sym->rev_dep.expr);
+  }
+
+  return sym->depends;
+}
+
+bool idepsym_expr(struct expr *e)
+{
+  struct expr *left, *right;
+  struct symbol *sym;
+
+  left = e->left.expr;
+  right = e->right.expr;
+  sym = e->left.sym;
+
+	switch (e->type) {
+	case E_SYMBOL:
+	case E_EQUAL:
+	case E_UNEQUAL:
+    return idepsym(sym);
+
+	case E_NOT:
+    return true;
+
+	case E_OR:
+    return idepsym_expr(left) || idepsym_expr(right);
+
+	case E_AND:
+    return idepsym_expr(left) && idepsym_expr(right);
+
+  default:
+    fprintf(stderr, "error: invalid expression type\n");
+    /* exit(1); */
+	}
+}
+
 bool is_symbol(struct symbol *sym)
 {
   struct property *st;
@@ -760,17 +1150,37 @@ void print_usage(void)
   printf("-h, --help\t\tdisplay this help message\n");
   printf("\n");
   printf("ACTIONS\n");
+  printf("--depsym\tprint all config vars that depend on arch\n");
+  printf("--idepsym\tsame as depsym, but boolean operations inverted\n");
+  printf("--tristate\tprint all tristate config vars that depend on arch\n");
+  printf("--bools\tprint all boolean config vars that depend on arch\n");
+  printf("--nonbools\tprint all non-boolean config vars that depend on arch\n");
   printf("--configs\tprint all config vars\n");
   printf("--kconfigs\tprint all config vars declared in kconfig files\n");
   printf("--menusyms\t"
          "print config vars declared in the Kconfig files (using menus)\n");
+  printf("--everyyes\t"
+         "output linux config files with all config vars set to yes\n");
   printf("--everyno\t"
          "output linux config files with all config vars set to no\n");
   printf("--everydef\t"
          "output linux config files with only root config vars set to yes\n");
+  printf("--everyyesi\t"
+         "output linux config files the inverse of everyyes\n");
   printf("--defaults\tprint all configuration variables that are defaults\n");
+  printf("--defdeps\t"
+         "print all defaults and config vars that only depend on defaults\n");
+  printf("--symdeps\t"
+         "for each config var, list the config vars on which it depends\n");
   printf("--extract\t"
          "extract constraints in kclause format\n");
+  printf("--autoconf-free\t"
+         "print booleans and tristates as free vars in autoconf.h format\n");
+  printf("--autoconf-feature-model\t"
+         "print booleans and tristate with constraints in autoconf.h format\n");
+  printf("--autoconf-cnf\t"
+         "print dependences as cnf clauses for a sat solver\n");
+  printf("--search VAR\tfind all config vars that depend on VAR\n");
   printf("--deps VAR\tprint direct and reverse dependencies for VAR\n");
   printf("--dump\t\tdump configuration variables\n");
   printf("\n");
@@ -801,15 +1211,28 @@ int main(int argc, char **argv)
   opterr = 0;
   while (1) {
     static struct option long_options[] = {
+      {"depsym", no_argument, &action, A_DEPSYM},
+      {"idepsym", no_argument, &action, A_IDEPSYM},
+      {"nonbools", no_argument, &action, A_NONBOOLS},
+      {"bools", no_argument, &action, A_BOOLS},
+      {"tristate", no_argument, &action, A_TRISTATE},
       {"configs", no_argument, &action, A_CONFIGS},
       {"kconfigs", no_argument, &action, A_KCONFIGS},
       {"menusyms", no_argument, &action, A_MENUSYMS},
       {"defaults", no_argument, &action, A_DEFAULTS},
+      {"symdeps", no_argument, &action, A_SYMDEPS},
+      {"defdeps", no_argument, &action, A_DEFDEPS},
+      {"everyyes", no_argument, &action, A_EVERYYES},
       {"everyno", no_argument, &action, A_EVERYNO},
       {"everydef", no_argument, &action, A_EVERYDEF},
+      {"everyyesi", no_argument, &action, A_EVERYYESI},
       {"forceoff", required_argument, 0, 'f'},
       {"forceoffall", required_argument, 0, 'a'},
+      {"autoconf-free", no_argument, &action, A_AUTOCONF_FREE},
+      {"autoconf-feature-model", no_argument, &action, A_AUTOCONF_FEATURE_MODEL},
+      {"autoconf-cnf", no_argument, &action, A_AUTOCONF_CNF},
       {"extract", no_argument, &action, A_EXTRACT},
+      {"search", required_argument, &action, A_SEARCH},
       {"deps", required_argument, &action ,A_DEPS},
       {"dump", no_argument, &action ,A_DUMP},
       {"Configure", no_argument, 0, 'C'},
@@ -940,6 +1363,15 @@ int main(int argc, char **argv)
   /* } */
 
   switch (action) {
+  case A_SEARCH:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      check_symbol(sym, action_arg);
+    break;
   case A_DEFAULTS:
     for_all_symbols(i, sym) {
       static bool def;
@@ -969,16 +1401,436 @@ int main(int argc, char **argv)
         printf("%s\n", sym->name);
     }
     break;
+  case A_EVERYYES:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+    for_all_symbols(i, sym)
+      idepsym(sym);
+    write_config(idepsym);
+    /* if (!bconf_parser) file_write_dep("include/config/auto.conf.cmd"); */
+    break;
   case A_EVERYNO:
     write_config(never);
     break;
   case A_EVERYDEF:
     write_config(is_default);
     break;
+  case A_EVERYYESI:
+    write_config(inverse);
+    break;
+  case A_SYMDEPS:
+    for_all_symbols(i, sym) {
+      int i2;
+      struct symbol *sym2;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      for_all_symbols(i2, sym2) {
+        sym->searched = false;
+      }
+
+      printf("%s=", sym->name);
+      print_deps(sym);
+      printf("\n");
+    }
+    break;
+  case A_DEFDEPS:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      defdeps(sym);
+
+    for_all_symbols(i, sym) {
+
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (dep)
+        printf("%s\n", sym->name);
+    }
+    break;
   case A_MENUSYMS:
     print_menusyms(rootmenu.list);
     break;
+  case A_DEPSYM:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      depsym(sym);
+
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (!dep)
+        printf("%s\n", sym->name);
+    }
+    break;
+  case A_IDEPSYM:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      idepsym(sym);
+
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (dep)
+        printf("%s\n", sym->name);
+    }
+    break;
+  case A_NONBOOLS:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      idepsym(sym);
+
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (dep && sym->type != S_BOOLEAN && sym->type != S_TRISTATE)
+        printf("%s\n", sym->name);
+    }
+    break;
+  case A_BOOLS:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      idepsym(sym);
+
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (dep && sym->type == S_BOOLEAN)
+        printf("%s\n", sym->name);
+    }
+    break;
+  case A_TRISTATE:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      idepsym(sym);
+
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (dep && sym->type == S_TRISTATE)
+        printf("%s\n", sym->name);
+    }
+    break;
+  case A_DEPS:
+    for_all_symbols(i, sym) {
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (strcmp(sym->name, action_arg) == 0) {
+        printf("dir_dep: ");
+        if (sym->dir_dep.expr)
+          print_expr(sym->dir_dep.expr, stdout, E_NONE);
+        printf("\n");
+
+        printf("rev_dep: ");
+        if (sym->rev_dep.expr)
+          print_expr(sym->rev_dep.expr, stdout, E_NONE);
+        printf("\n");
+
+        printf("in arch? %s\n", idepsym(sym) ? "yes" : "no");
+      }
+    }
+    break;
+  case A_AUTOCONF_FREE:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      idepsym(sym);
+
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (sym->type == S_TRISTATE || sym->type == S_BOOLEAN) {
+        if (dep) {
+          printf("#ifdef CONFIG_%s\n# define CONFIG_%s 1\n#else\n# undef CONFIG_%s\n#endif\n\n", sym->name, sym->name, sym->name);
+        } else {
+          // config var never defined
+          printf("#undef CONFIG_%s\n\n", sym->name);
+        }
+      }
+    }
+    break;
+  case A_AUTOCONF_FEATURE_MODEL:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      idepsym(sym);
+
+    // print all undefined config vars
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (sym->type == S_TRISTATE || sym->type == S_BOOLEAN) {
+        if (! dep) {
+          printf("#undef CONFIG_%s\n\n", sym->name);
+        }
+      }
+    }
+    
+    // print all root config vars
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (sym->type == S_TRISTATE || sym->type == S_BOOLEAN) {
+        if (dep && ! sym->dir_dep.expr) {
+            printf("#ifdef CONFIG_%s\n# define CONFIG_%s 1\n#else\n# undef CONFIG_%s\n#endif\n\n", sym->name, sym->name, sym->name);
+        }
+      }
+    }
+
+    // print all dependent config vars
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (sym->type == S_TRISTATE || sym->type == S_BOOLEAN) {
+        if (dep && sym->dir_dep.expr) {
+          printf("#if (");
+          if (sym->dir_dep.expr)
+            print_expr(sym->dir_dep.expr, stdout, E_NONE);
+          else
+            printf("0");
+          /* printf(") || ("); */
+          /* if (sym->rev_dep.expr) */
+          /*   print_expr(sym->rev_dep.expr, stdout, E_NONE); */
+          /* else */
+          /*   printf("0"); */
+          printf(")");
+          printf(" && (defined CONFIG_%s)", sym->name);
+          printf("\n");
+          printf("# define CONFIG_%s 1\n#else\n# undef CONFIG_%s\n#endif\n\n", sym->name, sym->name);
+        }
+      }
+    }
+    break;
+  case A_AUTOCONF_CNF:
+    for_all_symbols(i, sym) {
+      sym->searched = false;
+      sym->depends = false;
+    }
+
+    for_all_symbols(i, sym)
+      idepsym(sym);
+
+    // print all undefined config vars
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (sym->type == S_TRISTATE || sym->type == S_BOOLEAN) {
+        if (! dep) {
+          fprintf(stderr, "%s is always false\n", sym->name);
+          /* printf("#undef CONFIG_%s\n\n", sym->name); */
+        }
+      } else {
+        /* fprintf(stderr, "skipping %s\n", sym->name); */
+      }
+    }
+    
+    // print all root config vars
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (sym->type == S_TRISTATE || sym->type == S_BOOLEAN) {
+        if (dep && ! sym->dir_dep.expr) {
+          fprintf(stderr, "%s is a root\n", sym->name);
+            /* printf("#ifdef CONFIG_%s\n# define CONFIG_%s 1\n#else\n# undef CONFIG_%s\n#endif\n\n", sym->name, sym->name, sym->name); */
+        }
+      }
+    }
+
+    // print all dependent config vars
+    for_all_symbols(i, sym) {
+      bool dep = sym->depends;
+
+      if (sym_is_choice(sym) && sym->type == S_BOOLEAN) {
+        struct property *prop;
+        struct symbol *def_sym;
+        struct expr *e;
+
+        /* fprintf(stderr, "found boolean choice:"); */
+        prop = sym_get_choice_prop(sym);
+
+        // output mutually exclusive choices.  for each variable,
+        // represent an expression that says if "the config var is
+        // defined, then all the others are not", e.g., if A then
+        // !B!C..., i.e., !A or !B!C
+
+        expr_list_for_each_sym(prop->expr, e, def_sym) {
+          fprintf(stderr, " %s:", def_sym->name);
+          print_clause(def_sym->dir_dep.expr, stderr, E_NONE);
+          struct symbol *def_sym2;
+          struct expr *e2;
+          char *delim;
+
+          // also emit choice block's dependency on each choice
+          struct expr *choice_dep_expr;
+          choice_dep_expr = prop->menu->dep;
+          if (choice_dep_expr) {
+            printf("!CONFIG_%s || (", def_sym->name);
+            print_clause(choice_dep_expr, stdout, E_NONE);
+            printf(")\n");
+          }
+
+          fprintf(stdout, "!CONFIG_%s", def_sym->name);
+          delim = " || ";
+          expr_list_for_each_sym(prop->expr, e2, def_sym2) {
+            if (strcmp(def_sym->name, def_sym2->name) != 0) {
+              fprintf(stdout, "%s", delim);
+              delim = " && ";
+              fprintf(stdout, "!CONFIG_%s", def_sym2->name);
+            }
+          }
+          fprintf(stdout, "\n");
+        }
+        fprintf(stderr, "\n");
+      }
+      
+      if (!sym->name || strlen(sym->name) == 0)
+        continue;
+
+      if (sym->type == S_TRISTATE || sym->type == S_BOOLEAN) {
+        if (dep) {
+          if (sym->dir_dep.expr) {
+            /* // convert to cnf clauses */
+            /* if (sym_is_choice_value(sym)) { */
+            /*   print_choice_clauses(sym->dir_dep.expr, sym, stdout); */
+            /* } */
+            printf("!CONFIG_%s || (", sym->name);
+            print_clause(sym->dir_dep.expr, stdout, E_NONE);
+            printf(")\n");
+          }
+          /* if (sym->rev_dep.expr) { */
+          /*   // convert to cnf clauses */
+          /*   printf("!CONFIG_%s || (", sym->name); */
+          /*   print_clause(sym->rev_dep.expr, stdout, E_NONE); */
+          /*   printf(")\n"); */
+          /* } */
+          //if (1) continue;
+          struct property *st;
+          for_all_properties(sym, st, P_SELECT) {
+            // sym selects another config
+            if (! st->visible.expr) {
+              if (1) printf("!CONFIG_%s || CONFIG_%s\n",
+                     sym->name,
+                     st->expr->left.sym->name);
+            } else {
+              if (0) printf("!CONFIG_%s || CONFIG_%s\n",
+                     sym->name,
+                     st->expr->left.sym->name);
+              // convert to cnf clauses
+              if (1) {
+              /* printf("!(CONFIG_%s && (", sym->name); */
+              /* print_clause(st->visible.expr, stdout, E_NONE); */
+              /* printf(")) || CONFIG_%s\n", st->expr->left.sym->name); */
+                printf("!CONFIG_%s || CONFIG_%s || !(",
+                       sym->name,
+                       st->expr->left.sym->name);
+                print_clause(st->visible.expr, stdout, E_NONE);
+                printf(")\n");
+              }
+            }
+            /* printf("%s", st-> */
+          }
+          /* char delim = ' '; */
+          /* printf("CONFIG_%s selects", sym->name); */
+          /* struct property *st; */
+          /* for_all_properties(sym, st, P_SELECT) { */
+          /*   printf("%c", delim); */
+          /*   printf("CONFIG_%s", st->expr->left.sym->name); */
+          /*   printf("%s", st-> */
+          /*   delim = ','; */
+          /* } */
+          /* printf("\n"); */
+        }
+      } else {
+        /* fprintf(stderr, "skipping %s\n", sym->name); */
+      }
+    }
+    break;
   case A_EXTRACT:
+    // don't worry about selectability for extract
+    /* for_all_symbols(i, sym) { */
+    /*   sym->searched = false; */
+    /*   sym->depends = false; */
+    /* } */
+
+    /* for_all_symbols(i, sym) */
+    /*   idepsym(sym); */
+
+
+    // let kclause.py do this
+    /* // emit the boolean/tristate symbols.  add one for the root of the */
+    /* // feature model, necessary to create clauses for unconstrained */
+    /* // configuration variables */
+    /* #define SPECIAL_ROOT_NAME "SPECIAL_ROOT_VARIABLE" */
+    /* fprintf(output_fp, "bool %s\n", SPECIAL_ROOT_NAME); */
     for_all_symbols(i, sym) {
       if (!sym->name || strlen(sym->name) == 0)
         continue;
@@ -1264,8 +2116,7 @@ int main(int argc, char **argv)
           for_all_properties(sym, prop, P_SELECT) {
             // the current var itself is the var doing the select
             // prop->expr is the variable being selected
-            // prop->visible.expr is And(sym->dir_dept, select_dep) where select_dep
-            // is the dependency for select defined as "select 'selected' if 'select_dep'"
+            // prop->visible.expr is the "if ..." after the select
             fprintf(output_fp, "select ");
             // note: this assumes that prop->expr is only a single
             // variable name, which zconf.y guarantees
@@ -1279,7 +2130,6 @@ int main(int argc, char **argv)
             fprintf(output_fp, ")\n");
           }
 
-          // print the reverse dependency for this variable
           if (sym->rev_dep.expr) {
             no_dependencies = false;
             fprintf(output_fp, "rev_dep %s%s (", config_prefix, sym->name);
